# -*- coding: utf-8 -*-
"""

    ファイル名：   RequiredPackages.py

    処理内容：

        必須パッケージのインストール

"""

from __future__ import division, print_function, unicode_literals
import sys
import os
import platform
import shutil
from subprocess import call
from BasicUtils import clear_dirs_with_retry, rename_with_retry, home_dir

required_packages = [
    'numpy',
    'matplotlib',
    'pandas',
    'scipy',
    # 'pywin32',
    'PIL',
    'packaging',
    'nose',
    'coverage',
    'pypiwin32',
    ]

site_url = 'http://www.lfd.uci.edu/~gohlke/pythonlibs/'

python_version_short = '%d%d' % sys.version_info[0:2]

bitness_str_map = {
    '32bit' : 'win32',
    '64bit' : 'win_amd64',
    }

bitness_str = bitness_str_map.get( platform.architecture()[0] )
if not bitness_str: assert( False )

postfix = '-cp%s-none-%s.whl' % ( python_version_short, bitness_str )

print( 'postfix=', postfix )

wheel_file_map = {
    # Caveate! : web page may include non-ascii characters.
    # e.g. '‑'(U+2011)s in 'numpy‑1.9.3+mkl‑cp35‑none‑win_amd64.whl'
    # Do not simply copy and paste!
    'numpy'         : 'numpy-1.9.3+mkl{0}'  .format( postfix ),
    'matplotlib'    : 'matplotlib-1.5.0{0}' .format( postfix ),
    'pandas'        : 'pandas-0.17.1{0}'    .format( postfix ),
    'scipy'         : 'scipy-0.16.1{0}'     .format( postfix ),
    'PIL'           : 'Pillow-3.0.0{0}'     .format( postfix ),
    }

to_be_installed_packages = []

def current_platform():
    return ( platform.python_version(), platform.system(), platform.architecture()[0] )

def check_tested():
    current_platform_ = current_platform()

    print( 'Current platform is python %s on %s %s.' % ( current_platform_ ) )

    tested_platforms = [
        ( '3.5.1rc1', 'Windows', '64bit' ),
        ( '3.5.0', 'Windows', '64bit' ),
        ( '3.5.0', 'Windows', '32bit' ),
        ]

    assert( current_platform_ in tested_platforms )

def requirements_ok():

    for package in required_packages:
        print( 'importing', package )
        try:
            exec( 'import ' + package )
        except:
            print( '%s is not yet installed.' % ( package ) )
            # print( sys.exc_info[0] )
            to_be_installed_packages.append( package )

    if len( to_be_installed_packages ) == 0:
        return True

    install_packages()

    return False

def install():
    check_tested()

    assert( platform.system() == 'Windows' )
    try:
        import win32api
    except:
        print( "Can't check admin privileges." )
        print( "If you are sure, comment out the following 'exit()' and retry." )
        # exit()

    ret = call( "pip -V", shell=True )
    if ret == 0:
        print( 'pip is available.' )
    else:
        print( 'Install pip manually.' )
        exit()

    """
    # TODO: download automation
    'wget' is not enough because we can't get target file url.

    print( 'Installing wget' )
    ret = call( "pip install wget" )
    if ret == 0:
        pass
    else:
        print( "Can't install wget. Please make sure to get admin privileges." )
        exit()
    """

    download_dir = os.path.join( home_dir(), 'Downloads' )
    # print( 'download_dir=', download_dir )

    num_skipped_packages = 0
    num_installed_packages = 0
    for package in required_packages:
        print( 'Installing %s' % ( package ) )
        wheel_file = wheel_file_map.get( package )
        # print( 'wheel_file=%s' % ( wheel_file) )

        if wheel_file == None:
            print( 'We will try to simply pip install {0}'.format( package ))
            ret = call( 'pip install {0}'.format( package ), shell=True )
            assert( ret == 0 )
        else:
            wheel_path = os.path.join( download_dir, wheel_file )
            # print( 'wheel_path=', wheel_path )

            if os.path.exists( wheel_path ):
                print( '%s has been downloaded, ok.' % ( wheel_file ) )
            else:
                print( '%s has not yet been downloaded.' % ( wheel_file ) )
                print( 'Skip installing {0}.'.format( package ) )
                num_skipped_packages += 1
                continue

            ret = call( 'pip install {0}'.format( wheel_path ), shell=True )
            assert( ret == 0 )
        num_installed_packages += 1

    assert( num_skipped_packages == 0 )
    assert( num_installed_packages == len( required_packages ) )

    print( 'All required packages have been successfully installed.' )
